/**
 * @file main.cpp
 * @brief AJAX Graph example
 *
 * This example demostrates the code to create a webpage with a dynamic graph,
 *  generated by data from the Netburner device. Specifically, it has been
 *  designed to imitate the data formats found in log files, where ordering and
 *  frequency are varied between data sources.
 *
 *  The vast majority of the example is simply the support code needed to
 *  generate the random data. This code is located in the other support files.
 *  The locations that actually are relevant to the AJAX portion of the example
 *  are:
 *      main.cpp:
 *          getSensorData()
 *          DumpLogData()
 *      data.html:
 *          <!--CPPCALL getSensorData() -->
 *      index.html:
 *          Basically, all the Javascript.
 *
 * @author Netburner,  Inc.
 * @version 1.0
 * @date 2013-05-21
 */


#include <predef.h>
#include <stdio.h>
#include <startnet.h>
#include <init.h>
#include "datalog.h"
#include "datagenerator.h"

extern "C" {
void UserMain(void * pd);
void getHTMLTicks(int sock, PCSTR url);
}

const char * AppName="AJAXGraph";

/**
 * @brief getHTMLTicks
 *  Simply sends a string representation of the current TimeTick out
 *  the given socket/file descriptor
 *
 * @param sock file descriptor of the TCP socket for the HTTP stream
 * @param url Pointer to a string containing the URL requested
 */
void getHTMLTicks(int sock, PCSTR url) {
	char tick[40];
	siprintf(tick, "%d", TimeTick);
	writestring(sock, tick);
}

/**
 * @brief DumpLogData
 *  A function to be passed to the RingLog's Serialize function, which will serialize
 *  the data log to JSON objects inside an array
 *
 * @param item A pointer to a datastruct about to be serialized
 * @param args A file descriptor to the TCP socket of the HTTP stream
 */
void DumpLogData( dataStruct *item, void *args )
{
    char buf[80];
    int fd = (int) args;
    sprintf( buf, "{id:%d,timeStamp:%ld,value:%.4f},",
            item->sensorID, item->timeStamp, item->value );
    writestring( fd, buf );
}

/**
 * @brief getSensorData
 *  Is called by one of the AJAX methods to return the datalog in
 *  JSON form
 *
 * @param sock file descriptor for the TCP socket of the HTTP stream
 * @param url Pointer to a string containing the URL requested
 */
void getSensorData(int sock, PCSTR url)
{
    writestring( sock, "{entries:[" );
    dataLog.Serialize( DumpLogData, (void *)sock );
    writestring( sock, "]}" );
}

/**
 * @brief DumpAddr
 *  Another RingLog Serialize function. This will serialize the log to CSV format
 *
 * @param item A pointer to a datastruct about to be serialized
 * @param args A file descriptor to the stream to serialize to
 */
void DumpAddr( dataStruct *item, void *args )
{
    char buf[60];
    sprintf( buf, "%d,%ld,%.4f\r\n",
            item->sensorID, item->timeStamp, item->value );
    writestring( (int)args, buf );
}

void UserMain(void * pd)
{
    initWithWeb(); // Perform all the nice backend setup: network stack, DHCP, Webserver, etc.

    uint32_t sensorTick;
    uint32_t logTickOffset = SeedLog();
    uint32_t nextSensor1Tick = ((logTickOffset / SENSOR_1_PERIOD) + 1) * SENSOR_1_PERIOD;
    uint32_t nextSensor2Tick = ((logTickOffset / SENSOR_2_PERIOD) + 1) * SENSOR_2_PERIOD;

    iprintf("Application: %s\r\nNNDK Revision: %s\r\n",AppName,GetReleaseTag());
    while (1) {
        OSTimeDly(4);
        sensorTick = logTickOffset + TimeTick;

        // if it's time to do so, log the sensor value
        if (sensorTick > nextSensor1Tick) {
            LogSensor( 1, sensorTick );
            nextSensor1Tick += SENSOR_1_PERIOD;
        }
        if (sensorTick > nextSensor2Tick) {
            LogSensor( 2, sensorTick );
            nextSensor2Tick += SENSOR_2_PERIOD;
        }

        // output the datalog if the user wants
        if ( charavail() ) {
            getchar();
            dataLog.Serialize( DumpLogData, (void *)CurrentStdioFD(1) );
        }
    }
}
